﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace CMan
{
    //------------------------------------------------------------------------
    public partial class Form1 : Form
    {
        private const char COM_WHO = '@';
        private const char COM_GET = '?';
        private const char COM_SET = '=';
        private const char COM_AND = '&';
        private const char COM_OR  = '|';
        private const char COM_EX  = '^';

        private const byte REG_PORTB = 0;
        private const byte REG_PINB  = 1;
        private const byte REG_DDRB  = 2;
        private const byte REG_PORTC = 3;
        private const byte REG_PINC  = 4;
        private const byte REG_DDRC  = 5;

        private const int OFF_LED = 0;
        private const int SET_LED = 1;
        private const int TOG_LED = 2;

        private const int REC_IDLE = 0;
        private const int REC_READY = 1;
        private const int REC_ERR = 2;
        
        private const int SERIAL_PORT_OK = 0;
        private const int SERIAL_PORT_ERR = 1;
        private const int SERIAL_PORT_BAUD = 9600;
        private const int SERIAL_COM_DELAY = 100;

        static int gSerialStatus = SERIAL_PORT_ERR;
        char[] buff = new char[32];
        static int gReceiveStatus;
        static byte gReceiveValue;

        static byte gStatePort;
        static byte gStateStatusLed;
        public static byte gDirectionMask;
        public static byte gPullupMask;
        static byte gStateInput;


        //Form form2 = new Form2();
        public static byte gChannel;

        private const string url = "http://sklep.avt.pl";
        public const string filePath = "cmancfg.txt";
        private const string fileDefaultSource = "FF \r\n00 \r\nWyjście1 \r\nWyjście2 \r\nWyjście3 \r\nWyjście4 \r\nWyjście5 \r\nWyjście6 \r\nWyjście7 \r\nWyjście8 \r\n";
        public const int FILE_NOMOFLINES = 10;

        private const byte NUM_OF_CHANNELS = 9;
        Panel[] MyPanels = new Panel[NUM_OF_CHANNELS];
        public static Label[] MyLabels = new Label[NUM_OF_CHANNELS];

        //------------------------------------------------------------------------
        public Form1()
        {
            InitializeComponent();
            
            string[] portarray = System.IO.Ports.SerialPort.GetPortNames();
            comBox.Items.AddRange(portarray);
            comBox.SelectedIndex = 0;

            MyPanels[0] = panel1;
            MyPanels[1] = panel2;
            MyPanels[2] = panel3;
            MyPanels[3] = panel4;
            MyPanels[4] = panel5;
            MyPanels[5] = panel6;
            MyPanels[6] = panel7;
            MyPanels[7] = panel8;
            MyPanels[8] = panel9;

            MyLabels[0] = label1;
            MyLabels[1] = label2;
            MyLabels[2] = label3;
            MyLabels[3] = label4;
            MyLabels[4] = label5;
            MyLabels[5] = label6;
            MyLabels[6] = label7;
            MyLabels[7] = label8;

            FileInit();
        }

        //------------------------------------------------------------------------
        private void AVTpictureBox_MouseClick(object sender, MouseEventArgs e)
        {
            System.Diagnostics.Process.Start(url);
        }

        //------------------------------------------------------------------------
        private void comBox_SelectionChangeCommitted(object sender, EventArgs e)
        {
            serialPort.Close();
            serialPort.PortName = comBox.GetItemText(comBox.SelectedItem);
            serialPort.BaudRate = SERIAL_PORT_BAUD;
            serialPort.Close();
            //------------------------------------
            try
            {
                serialPort.Open();
                gSerialStatus = SERIAL_PORT_OK;
            }
            catch
            {
                gSerialStatus = SERIAL_PORT_ERR;
            }

            //------------------------------------
            if (gSerialStatus == SERIAL_PORT_OK)
            {
                ComGroup.Text = "Połączony ";
                BoxEnable();
                
                SendLabel.Text = null;
                SendCommand(gDirectionMask, REG_DDRB, COM_SET);
                System.Threading.Thread.Sleep(SERIAL_COM_DELAY);
                SendCommand(gDirectionMask, REG_DDRC, COM_SET);
                System.Threading.Thread.Sleep(SERIAL_COM_DELAY);

                OutputStateGet();
                PullupAdd();
                gStateStatusLed = 1;

                SendCommand(gStatePort, REG_PORTB, COM_SET);
                System.Threading.Thread.Sleep(SERIAL_COM_DELAY);
                SendCommand(gStatePort, REG_PORTC, COM_SET);
                System.Threading.Thread.Sleep(SERIAL_COM_DELAY);
                System.Threading.Thread.Sleep(SERIAL_COM_DELAY);

                OutputStateDisplay();
                InputStateDisplay();

                timer1.Start();
            }
            else 
            {
                ComGroup.Text = "Niepowodzenie...";
                //ComGroup.ForeColor = Color.Red;
                BoxDisable();
                MessageBox.Show("Nie udało się otworzyć portu...");
            }
        }

        //------------------------------------------------------------------------
        private void BoxEnable()
        {
            groupBox1.Enabled = true;
            groupBox2.Enabled = true;
            groupBox3.Enabled = true;
        }
        //------------------------------------------------------------------------
        private void BoxDisable()
        {
            groupBox1.Enabled = false;
            groupBox2.Enabled = false;
            groupBox3.Enabled = false;
        }

        //------------------------------------------------------------------------
        private void OutputStateChange(int channel)
        {

            if ((gDirectionMask & (1 << channel)) > 0)
            {
                if ((gStatePort & (1 << channel)) == 0)
                    gStatePort += (byte)(1 << channel);
                else
                    gStatePort -= (byte)(1 << channel);
            }
            else
            {
                if ((gStatePort & (1 << channel)) > 0)
                    gStatePort -= (byte)(1 << channel);
            }

            PullupAdd();
            
            SendLabel.Text = null;     
            if (channel < 6)
            {
                SendCommand((byte)gStatePort, REG_PORTB, COM_SET);
            }

            if ((channel >= 6) && (channel <= 7))
            {
                SendCommand((byte)gStatePort, REG_PORTC, COM_SET);
            }

            OutputStateDisplay();
        }
 
        //------------------------------------------------------------------------
        private void PullupAdd()
        {
            int i = 0;
            while (i < 8)
            {
                if ((gDirectionMask & (1 << i)) == 0)
                {
                    if ((gPullupMask & (1 << i)) > 0)
                    {
                        gStatePort |= (byte)(1 << i);
                    }
                }
                i++;
            }
        }

        //------------------------------------------------------------------------
        private void LedStateChange(int action)
        {
            if (action == OFF_LED)        //off
            { 
                gStateStatusLed  = 0;
            }
            if (action == SET_LED)        //set
            {
                gStateStatusLed = 1;
            }
            if (action == TOG_LED)        //toggle
            {
                if (gStateStatusLed > 0)
                    gStateStatusLed = 0;
                else
                    gStateStatusLed = 1;
            }

            if (gStateStatusLed > 0)
                panel9.BackgroundImage = ((System.Drawing.Image)(Properties.Resources.green_led_on));
            else
                panel9.BackgroundImage = ((System.Drawing.Image)(Properties.Resources.green_led_off));

            SendLabel.Text = null;
            SendCommand((byte)gStatePort, REG_PORTC, COM_SET);    
        }
        //------------------------------------------------------------------------
        private void OutputStateDisplay()
        {
            Panel LocalControl = new Panel();
            int i = 0;

            while (i < (NUM_OF_CHANNELS - 1))
            {
                LocalControl = MyPanels[i];

                if ((gDirectionMask & (1 << i)) > 0)
                {
                    if ((gStatePort & (1 << i)) > 0)
                        LocalControl.BackgroundImage = ((System.Drawing.Image)(Properties.Resources.led_on));
                    else
                        LocalControl.BackgroundImage = ((System.Drawing.Image)(Properties.Resources.led_off));
                }
                i++;
            }

            LocalControl = MyPanels[i];
            if (gStateStatusLed > 0)
                LocalControl.BackgroundImage = ((System.Drawing.Image)(Properties.Resources.green_led_on));
            else
                LocalControl.BackgroundImage = ((System.Drawing.Image)(Properties.Resources.green_led_off));
        }
        //------------------------------------------------------------------------
        private void OutputStateGet()
        {
            InputStateGet();
            SendLabel.Text = Convert.ToString(gStateInput);
            gStatePort = gStateInput;
        }

        //------------------------------------------------------------------------
        private void panel9_MouseClick(object sender, MouseEventArgs e)
        {
            LedStateChange(TOG_LED);
        }

        //------------------------------------------------------------------------
        private void panel1_MouseClick(object sender, MouseEventArgs e)
        {
            OutputStateChange(0);
        }
        private void panel2_MouseClick(object sender, MouseEventArgs e)
        {
            OutputStateChange(1);
        }
        private void panel3_MouseClick(object sender, MouseEventArgs e)
        {
            OutputStateChange(2);
        }
        private void panel4_MouseClick(object sender, MouseEventArgs e)
        {
            OutputStateChange(3);
        }
        private void panel5_MouseClick(object sender, MouseEventArgs e)
        {
            OutputStateChange(4);
        }
        private void panel6_MouseClick(object sender, MouseEventArgs e)
        {
            OutputStateChange(5);
        }
        private void panel7_MouseClick(object sender, MouseEventArgs e)
        {
            OutputStateChange(6);
        }
        private void panel8_MouseClick(object sender, MouseEventArgs e)
        {
            OutputStateChange(7);
        }

        //------------------------------------------------------------------------
        private void SendCommand(byte data, byte reg, char command)
        {
            int temp;
            int i;
            i = 0;

            timer1.Stop(); 
            
            serialPort.DiscardInBuffer();
            
            //-----------------------------------
            if (reg <= REG_DDRB)
            {
                data &= 0x3F;
            }
            else
            {
                data >>= 6; //only higher 2 bits
                if (reg == REG_DDRC)
                    data += 4;
                else
                {
                    if (gStateStatusLed > 0) data += 4;
                }
            }

            //-----------------------------------
            temp = (data / 16);
            if (temp < 10)
                temp += '0';
            else
            {
                temp -= 10;
                temp += 'A';
            }
            buff[i] = (char)temp; i++;

            temp = (data % 16);
            if (temp < 10)
                temp += '0';
            else
            {
                temp -= 10;
                temp += 'A';
            }
            buff[1] = (char)temp; i++;
            buff[2] = ' '; i++;

            //-----------------------------------
            if (reg == 0)
            {
                buff[i] = 'p'; i++; buff[i] = 'o'; i++; buff[i] = 'r'; i++; buff[i] = 't'; i++; buff[i] = 'b'; i++;
            }
            if (reg == 1)
            {
                buff[i] = 'p'; i++; buff[i] = 'i'; i++; buff[i] = 'n'; i++; buff[i] = 'b'; i++;
            }
            if (reg == 2)
            {
                buff[i] = 'd'; i++; buff[i] = 'd'; i++; buff[i] = 'r'; i++; buff[i] = 'b'; i++;
            }
            if (reg == 3)
            {
                buff[i] = 'p'; i++; buff[i] = 'o'; i++; buff[i] = 'r'; i++; buff[i] = 't'; i++; buff[i] = 'c'; i++;
            }
            if (reg == 4)
            {
                buff[i] = 'p'; i++; buff[i] = 'i'; i++; buff[i] = 'n'; i++; buff[i] = 'c'; i++;
            }
            if (reg == 5)
            {
                buff[i] = 'd'; i++; buff[i] = 'd'; i++; buff[i] = 'r'; i++; buff[i] = 'c'; i++;
            }

            //-----------------------------------
            buff[i] = ' '; i++;
            buff[i] = command; i++;
            buff[i] = '\r'; i++;
            buff[i] = '\n'; i++;
            buff[i] = (char)0; i++;
          
            //-----------------------------------
            try
            {
                serialPort.Write(buff, 0, i);
                SendLabel.Text += new string(buff);
                if (AutoCheckBox.Checked == true)
                    timer1.Start();
                else
                    timer1.Stop();
            }
            catch
            {
                BoxDisable();
                //SendLabel.Text = "Błąd...";
                MessageBox.Show("Komuniakcja nie powiodła sie...");
                timer1.Stop();
                this.Close();
            }
        }

        //------------------------------------------------------------------------
        private void ReadButton_Click(object sender, EventArgs e)
        {
            InputStateGet();
            InputStateDisplay();
        }

        //------------------------------------------------------------------------
        private void InputStateGet()
        {
            SendLabel.Text = null;
            SendCommand((byte)0x3F, REG_PINB, COM_GET);
            System.Threading.Thread.Sleep(SERIAL_COM_DELAY);

            if (gReceiveStatus == REC_READY)
            { 
                gStateInput = (byte)(gReceiveValue & 0x3F);
                gReceiveStatus = REC_IDLE;

                SendCommand((byte)0xC0, REG_PINC, COM_GET);
                System.Threading.Thread.Sleep(SERIAL_COM_DELAY);

                if (gReceiveStatus == REC_READY)
                {
                    if ((gReceiveValue & 0x01) > 0) gStateInput += 0x40;
                    if ((gReceiveValue & 0x02) > 0) gStateInput += 0x80;
                    
                    gStateStatusLed = 0;
                    if ((gReceiveValue & 0x04) > 0) gStateStatusLed = SET_LED;
                    
                    gReceiveStatus = REC_IDLE;
                }
            }
            gReceiveStatus = REC_IDLE;
        }

        //------------------------------------------------------------------------
        private void InputStateDisplay()
        {
            Panel LocalControl = new Panel();
            int i = 0;

            while (i < (NUM_OF_CHANNELS - 1))
            {
                LocalControl = MyPanels[i];
                if ((gDirectionMask & (1 << i)) == 0)
                {
                    if ((gStateInput & (1 << i)) > 0)
                        LocalControl.BackgroundImage = ((System.Drawing.Image)(Properties.Resources.led_on));
                    else
                        LocalControl.BackgroundImage = ((System.Drawing.Image)(Properties.Resources.led_off));                                  
                }
                i++; 
            }
        }

        //------------------------------------------------------------------------
        private void serialPort_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            byte value;
            string rxString;

            if (serialPort.BytesToRead > 2)
            {
                if (gReceiveStatus == REC_IDLE)
                {
                    rxString = serialPort.ReadExisting();

                    gReceiveStatus = REC_READY;
                    value = 0;

                    if ((rxString[0] >= '0') && (rxString[0] <= '9'))
                    {
                        value += (byte)((rxString[0] - '0') << 4);
                    }
                    else if ((rxString[0] >= 'A') && (rxString[0] <= 'F'))
                    {
                        value += (byte)(((rxString[0] - 'A') + 10) << 4);
                    }
                    else
                        gReceiveStatus = REC_ERR;

                    if ((rxString[1] >= '0') && (rxString[1] <= '9'))
                    {
                        value += (byte)(rxString[1] - '0');
                    }
                    else if ((rxString[1] >= 'A') && (rxString[1] <= 'F'))
                    {
                        value += (byte)((rxString[1] - 'A') + 10);
                    }
                    else
                        gReceiveStatus = REC_ERR;

                    if (gReceiveStatus == REC_READY)
                        gReceiveValue = value;
                    else
                        gReceiveStatus = REC_IDLE;
                }
            }
            serialPort.DiscardInBuffer();

            /*if (gReceiveStatus == REC_READY)
            {
                this.Invoke(new EventHandler(StateDsip));
                gReceiveStatus = REC_IDLE; 
            }*/
        }

        //------------------------------------------------------------------------
        /*private void StateDsip(object sender, EventArgs e)
        {
            //SendLabel.Text = null;
            //SendLabel.Text += new string(buff);
            SendLabel.Text = Convert.ToString(gReceiveValue);
        }*/

        //------------------------------------------------------------------------
        private int Hex2val(string hex)
        {
            int val = 0;
            if ((hex[0] >= '0') && (hex[0] <= '9')) val += ((hex[0] - '0') << 4);
            if ((hex[0] >= 'A') && (hex[0] <= 'F')) val += (((hex[0] - 'A') + 10) << 4);
            if ((hex[1] >= '0') && (hex[1] <= '9')) val += (hex[1] - '0');
            if ((hex[1] >= 'A') && (hex[1] <= 'F')) val += ((hex[1] - 'A') + 10);
            return val;
        }
        //------------------------------------------------------------------------
        private void FileInit()
        {
            int index = 0;
            string line;

            if (!File.Exists(filePath))
            {
                File.WriteAllText(filePath, fileDefaultSource);
            }

            System.IO.StreamReader file = new System.IO.StreamReader(@filePath);
            
            line = file.ReadLine();
            gDirectionMask = (byte)Hex2val(line);     
            SendLabel.Text += Convert.ToString(gDirectionMask);

            
            line = file.ReadLine();
            gPullupMask = (byte)Hex2val(line);
            SendLabel.Text += Convert.ToString(gPullupMask);

            index = 0;
            while (index < (FILE_NOMOFLINES - 2))
            {
                MyLabels[index].Text = file.ReadLine();
                index++;
            }

            file.Close();
        }

        //------------------------------------------------------------------------
        private void AutoCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (AutoCheckBox.Checked == true)
            {
                ReadButton.Enabled = false;
                timer1.Start();
            }
            else
            {
                timer1.Stop();
                ReadButton.Enabled = true;
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            InputStateGet();
            InputStateDisplay();
        }
    }
}
